@@ -33,7 +33,7 @@ from .const import (
     ZON_ROLE_MAP,
     __dev_mode__,
 )
-from .devices_base import BatteryState, DeviceHeat, Fakeable, check_faking_enabled
+from .devices_base import BatteryState, DeviceHeat, Fakeable
 from .entity_base import Entity, Parent, class_by_attr
 from .helpers import shrink
 from .protocol import Address, Command, Message, Priority
@@ -52,7 +52,7 @@ from .protocol.opentherm import (
     OtMsgType,
 )
 from .protocol.ramses import CODES_HEAT_ONLY, CODES_ONLY_FROM_CTL, CODES_SCHEMA
-from .schema import SCHEMA_SYS, SZ_ACTUATORS, SZ_CIRCUITS
+from .schema import SCH_SYS, SZ_ACTUATORS, SZ_CIRCUITS
 
 # skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
@@ -151,20 +151,23 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
 
 
 DEV_MODE = __dev_mode__  # and False
-_OTB_MODE = False  # use OT (3220s) in favour of RAMSES
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-class Actuator(Fakeable, DeviceHeat):  # 3EF0, 3EF1
-    # RP --- 10:048122 18:140805 --:------ 3EF1 007 007FFF-003C2A10       # only RP, always 7FFF
-    # RP --- 13:109598 18:199952 --:------ 3EF1 007 0001B8-01B800FF       # only RP
+class Actuator(Fakeable, DeviceHeat):  # 3EF0, 3EF1 (for 10:/13:)
+    #  I --- 13:109598 --:------ 13:109598 3EF0 003 00C8FF                # event-driven, 00/C8
+    # RP --- 13:109598 18:002563 --:------ 0008 002 00C8                  # 00/C8, as abobe
+    # RP --- 13:109598 18:002563 --:------ 3EF1 007 0000BF-00BFC8FF       # 00/C8, as above
 
-    # RP --- 10:047707 18:199952 --:------ 3EF0 009 001110-0A00FF-033100  # only RP
-    # RP --- 10:138926 34:010253 --:------ 3EF0 006 002E11-0000FF         # only RP
-    #  I --- 13:209679 --:------ 13:209679 3EF0 003 00C8FF                # only  I
+    # RP --- 10:048122 18:140805 --:------ 3EF1 007 007FFF-003C2A10       # 10:s only RP, always 7FFF
+    # RP --- 13:109598 18:199952 --:------ 3EF1 007 0001B8-01B800FF       # 13:s only RP
+
+    # RP --- 10:047707 18:199952 --:------ 3EF0 009 001110-0A00FF-033100  # 10:s only RP
+    # RP --- 10:138926 34:010253 --:------ 3EF0 006 002E11-0000FF         # 10:s only RP
+    #  I --- 13:209679 --:------ 13:209679 3EF0 003 00C8FF                # 13:s only  I
 
     ACTUATOR_CYCLE = "actuator_cycle"
     ACTUATOR_ENABLED = "actuator_enabled"  # boolean
@@ -177,13 +180,14 @@ class Actuator(Fakeable, DeviceHeat):  # 3EF0, 3EF1
         if isinstance(self, OtbGateway):
             return
 
-        if False and (
+        if (
             msg.code == _3EF0
-            and msg.verb == I_
+            and msg.verb == I_  # will be a 13:
             and not self._faked
             and not self._gwy.config.disable_discovery
             and not self._gwy.config.disable_sending
         ):
+            # self._make_cmd(_0008, qos={SZ_PRIORITY: Priority.LOW, SZ_RETRIES: 1})
             self._make_cmd(_3EF1, qos={SZ_PRIORITY: Priority.LOW, SZ_RETRIES: 1})
 
     @property
@@ -254,9 +258,11 @@ class Weather(Fakeable, DeviceHeat):  # 0002
     def temperature(self) -> Optional[float]:  # 0002
         return self._msg_value(_0002, key=self.TEMPERATURE)
 
+    # @check_faking_enabled
     @temperature.setter
-    @check_faking_enabled
     def temperature(self, value) -> None:  # 0002
+        if not self._faked:
+            raise RuntimeError(f"Faking is not enabled for {self}")
         cmd = Command.put_outdoor_temp(self.id, value)
         # cmd = Command.put_zone_temp(
         #     self._gwy.hgi.id if self == self._gwy.hgi._faked_thm else self.id, value
@@ -272,6 +278,14 @@ class Weather(Fakeable, DeviceHeat):  # 0002
 
 
 class RelayDemand(Fakeable, DeviceHeat):  # 0008
+    # Some either 00/C8, others 00-C8
+    #  I --- 01:145038 --:------ 01:145038 0008 002 0314  # zone valve zone (not electric?)
+    #  I --- 01:145038 --:------ 01:145038 0008 002 F914  # DHW valve
+    #  I --- 01:054173 --:------ 01:054173 0008 002 FA00  # DHW valve
+    #  I --- 01:145038 --:------ 01:145038 0008 002 FC14  # appliance_relay
+
+    # RP --- 13:109598 18:199952 --:------ 0008 002 0000
+    # RP --- 13:109598 18:199952 --:------ 0008 002 00C8
 
     RELAY_DEMAND = SZ_RELAY_DEMAND  # percentage (0.0-1.0)
 
@@ -279,7 +293,7 @@ class RelayDemand(Fakeable, DeviceHeat):  # 0008
         super()._setup_discovery_tasks()
 
         if not self._faked:  # discover_flag & Discover.STATUS and
-            self._add_discovery_task(Command.get_relay_demand(self.id), 60 * 60 * 5)
+            self._add_discovery_task(Command.get_relay_demand(self.id), 60 * 15)
 
     def _handle_msg(self, msg: Message) -> None:  # NOTE: active
         if msg.src.id == self.id:
@@ -367,9 +381,11 @@ class DhwTemperature(Fakeable, DeviceHeat):  # 1260
     def temperature(self) -> Optional[float]:  # 1260
         return self._msg_value(_1260, key=self.TEMPERATURE)
 
+    # @check_faking_enabled
     @temperature.setter
-    @check_faking_enabled
     def temperature(self, value) -> None:  # 1260
+        if not self._faked:
+            raise RuntimeError(f"Faking is not enabled for {self}")
         self._send_cmd(Command.put_dhw_temp(value))
         # lf._send_cmd(Command.get_dhw_temp(self.ctl.id, self.zone.idx))
 
@@ -400,9 +416,11 @@ class Temperature(Fakeable, DeviceHeat):  # 30C9
     def temperature(self) -> Optional[float]:  # 30C9
         return self._msg_value(_30C9, key=self.TEMPERATURE)
 
+    # @check_faking_enabled
     @temperature.setter
-    @check_faking_enabled
     def temperature(self, value) -> None:  # 30C9
+        if not self._faked:
+            raise RuntimeError(f"Faking is not enabled for {self}")
         self._send_cmd(Command.put_sensor_temp(self.id, value))
         # lf._send_cmd(Command.get_zone_temp(self.ctl.id, self.zone.idx))
 
@@ -451,7 +469,7 @@ class Controller(DeviceHeat):  # CTL (01):
 
             from .systems import zx_system_factory
 
-            schema = shrink(SCHEMA_SYS(schema))
+            schema = shrink(SCH_SYS(schema))
 
             if not self.tcs:
                 self.tcs = zx_system_factory(self, msg=msg, **schema)
@@ -602,7 +620,7 @@ class UfhController(Parent, DeviceHeat):  # UFC (02):
         If a circuit is created, attach it to this UFC.
         """
 
-        schema = {}  # shrink(SCHEMA_CCT(schema))
+        schema = {}  # shrink(SCH_CCT(schema))
 
         cct = self.child_by_id.get(cct_idx)
         if not cct:
@@ -768,19 +786,19 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
                 self._add_discovery_task(_mk_cmd(RQ, code, "00", self.id), 60)
 
         for m in SCHEMA_MSG_IDS:  # From OT v2.2: version numbers
-            if _OTB_MODE or m not in self.OT_TO_RAMSES:
+            if self._gwy.config.use_native_ot or m not in self.OT_TO_RAMSES:
                 self._add_discovery_task(
                     Command.get_opentherm_data(self.id, m), 60 * 60 * 24, delay=60 * 3
                 )
 
         for m in PARAMS_MSG_IDS:  # or L/T state
-            if _OTB_MODE or m not in self.OT_TO_RAMSES:
+            if self._gwy.config.use_native_ot or m not in self.OT_TO_RAMSES:
                 self._add_discovery_task(
                     Command.get_opentherm_data(self.id, m), 60 * 60, delay=90
                 )
 
         for msg_id in STATUS_MSG_IDS:
-            if _OTB_MODE or m not in self.OT_TO_RAMSES:
+            if self._gwy.config.use_native_ot or m not in self.OT_TO_RAMSES:
                 self._add_discovery_task(
                     Command.get_opentherm_data(self.id, msg_id), 60 * 5, delay=15
                 )
@@ -789,7 +807,7 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
         self._add_discovery_task(_mk_cmd(RQ, _2401, "00", self.id), 60 * 5)
         self._add_discovery_task(_mk_cmd(RQ, _3EF0, "00", self.id), 60 * 5)
 
-        if _OTB_MODE:
+        if self._gwy.config.use_native_ot:
             return
 
         for code in [v for k, v in self.OT_TO_RAMSES.items() if k in PARAMS_MSG_IDS]:
@@ -832,9 +850,11 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
             if msg_id != "73":
                 self._send_cmd(Command.get_opentherm_data(self.id, "73"))  # oem code
 
-        # TODO: this is development code - will be rationalised, eventually
-        if _OTB_MODE and (code := self.OT_TO_RAMSES.get(msg_id)):
-            self._send_cmd(_mk_cmd(RQ, code, "00", self.id))
+            # TODO: this is development code - will be rationalised, eventually
+            if self._gwy.config.use_native_ot and (
+                code := self.OT_TO_RAMSES.get(msg_id)
+            ):
+                self._send_cmd(_mk_cmd(RQ, code, "00", self.id))
 
         if msg._pkt.payload[6:] == "47AB" or msg._pkt.payload[4:] == "121980":
             if msg_id not in self._msgs_ot_supported:
@@ -929,10 +949,6 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
     def bit_6_6(self) -> Optional[bool]:  # 3EF0 ?dhw_enabled (byte 3, only R8820A?)
         return self._msg_flag(_3EF0, "_flags_3", 6)
 
-    @property
-    def oem_code(self) -> Optional[float]:  # 3220/73
-        return self._ot_msg_value("73")
-
     @property
     def percent(self) -> Optional[float]:  # 2401 - WIP
         return self._msg_value(_2401, key="_percent_3")
@@ -942,26 +958,26 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
         return self._msg_value(_2401, key="_value_2")
 
     @property
-    def boiler_output_temp(self) -> Optional[float]:  # 3200 (3220/19)
-        if _OTB_MODE:
+    def boiler_output_temp(self) -> Optional[float]:  # 3220|19, or 3200
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("19")
         return self._msg_value(_3200, key=SZ_TEMPERATURE)
 
     @property
-    def boiler_return_temp(self) -> Optional[float]:  # 3210 (3220/1C)
-        if _OTB_MODE:
+    def boiler_return_temp(self) -> Optional[float]:  # 3220|1C, or 3210
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("1C")
         return self._msg_value(_3210, key=SZ_TEMPERATURE)
 
     @property
-    def boiler_setpoint(self) -> Optional[float]:  # 22D9 (3220/01)
-        if _OTB_MODE:
+    def boiler_setpoint(self) -> Optional[float]:  # 3220|01, or 22D9
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("01")
         return self._msg_value(_22D9, key=SZ_SETPOINT)
 
     @property
-    def ch_max_setpoint(self) -> Optional[float]:  # 1081 (3220/39)
-        if _OTB_MODE:
+    def ch_max_setpoint(self) -> Optional[float]:  # 3220|39, or 1081
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("39")
         return self._msg_value(_1081, key=SZ_SETPOINT)
 
@@ -970,33 +986,45 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
         return self._msg_value(_3EF0, key="ch_setpoint")
 
     @property
-    def ch_water_pressure(self) -> Optional[float]:  # 1300 (3220/12)
-        if _OTB_MODE:
+    def ch_water_pressure(self) -> Optional[float]:  # 3220|12, or 1300
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("12")
         result = self._msg_value(_1300, key=SZ_PRESSURE)
         return None if result == 25.5 else result  # HACK: to make more rigourous
 
     @property
-    def dhw_flow_rate(self) -> Optional[float]:  # 12F0 (3220/13)
-        if _OTB_MODE:
+    def dhw_flow_rate(self) -> Optional[float]:  # 3220|13, or 12F0
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("13")
         return self._msg_value(_12F0, key="dhw_flow_rate")
 
     @property
-    def dhw_setpoint(self) -> Optional[float]:  # 10A0 (3220/38)
-        if _OTB_MODE:
+    def dhw_setpoint(self) -> Optional[float]:  # 3220|38, or 10A0
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("38")
         return self._msg_value(_10A0, key=SZ_SETPOINT)
 
     @property
-    def dhw_temp(self) -> Optional[float]:  # 1260 (3220/1A)
-        if _OTB_MODE:
+    def dhw_temp(self) -> Optional[float]:  # 3220|1A, or 1260
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("1A")
         return self._msg_value(_1260, key=SZ_TEMPERATURE)
 
     @property
-    def outside_temp(self) -> Optional[float]:  # 1290 (3220/1B)
-        if _OTB_MODE:
+    def max_rel_modulation(
+        self,
+    ) -> Optional[float]:  # 3220|0E, or 3EF0 (byte 8, only R8820A?)
+        if self._gwy.config.use_native_ot:
+            return self._ot_msg_value("0E")  # needs confirming
+        return self._msg_value(_3EF0, key="max_rel_modulation")
+
+    @property
+    def oem_code(self) -> Optional[float]:  # 3220|73
+        return self._ot_msg_value("73")
+
+    @property
+    def outside_temp(self) -> Optional[float]:  # 3220|1B, 1290
+        if self._gwy.config.use_native_ot:
             return self._ot_msg_value("1B")
         return self._msg_value(_1290, key=SZ_TEMPERATURE)
 
@@ -1006,55 +1034,49 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
         return self._msg_value((_3EF0, _3EF1), key=self.MODULATION_LEVEL)
 
     @property
-    def rel_modulation_level_ot(self) -> Optional[float]:  # 3220/11
+    def rel_modulation_level_ot(self) -> Optional[float]:  # 3220|11
         """Return the relative modulation level from OpenTherm."""
         return self._ot_msg_value("11")
 
     @property
-    def max_rel_modulation(self) -> Optional[float]:  # 3EF0 (byte 8, only R8820A?)
-        return self._msg_value(_3EF0, key="max_rel_modulation")
+    def ch_active(self) -> Optional[bool]:  # 3220|00, or 3EF0 (byte 3, only R8820A?)
+        if self._gwy.config.use_native_ot:
+            return self._ot_msg_flag("00", 8 + 1)
+        return self._msg_value(_3EF0, key="ch_active")
 
     @property
-    def ch_active(self) -> Optional[bool]:  # 3220/00, or 3EF0 (byte 3, only R8820A?)
-        if not _OTB_MODE:
-            return self._msg_value(_3EF0, key="ch_active")
-        return self._ot_msg_flag("00", 8 + 1) if _OTB_MODE else super().ch_active
+    def ch_enabled(self) -> Optional[bool]:  # 3220|00, or 3EF0 (byte 6, only R8820A?)
+        if self._gwy.config.use_native_ot:
+            return self._ot_msg_flag("00", 0)
+        return self._msg_value(_3EF0, key="ch_enabled")
 
     @property
-    def ch_enabled(self) -> Optional[bool]:  # 3220/00, or 3EF0 (byte 6, only R8820A?)
-        if not _OTB_MODE:
-            return self._msg_value(_3EF0, key="ch_enabled")
-        return self._ot_msg_flag("00", 0) if _OTB_MODE else super().ch_enabled
+    def dhw_active(self) -> Optional[bool]:  # 3220|00, or 3EF0 (byte 3, only OTB)
+        if self._gwy.config.use_native_ot:
+            return self._ot_msg_flag("00", 8 + 2)
+        return self._msg_value(_3EF0, key="dhw_active")
 
     @property
-    def dhw_active(self) -> Optional[bool]:  # 3220/00, or 3EF0 (byte 3, only OTB)
-        if not _OTB_MODE:
-            return self._msg_value(_3EF0, key="dhw_active")
-        return self._ot_msg_flag("00", 8 + 2) if _OTB_MODE else super().dhw_active
+    def dhw_enabled(self) -> Optional[bool]:  # 3220|00
+        return self._ot_msg_flag("00", 1)
 
     @property
-    def dhw_enabled(self) -> Optional[bool]:  # 3220/00
-        return self._ot_msg_flag("00", 1)  # if _OTB_MODE else None  # TODO: super().xxx
+    def flame_active(self) -> Optional[bool]:  # 3220|00, or 3EF0 (byte 3, only OTB)
+        if not self._gwy.config.use_native_ot:
+            return self._ot_msg_flag("00", 8 + 3)
+        return self._msg_value(_3EF0, key="flame_active")
 
     @property
-    def flame_active(self) -> Optional[bool]:  # 3220/00, or 3EF0 (byte 3, only OTB)
-        if not _OTB_MODE:
-            return self._msg_value(_3EF0, key="flame_active")
-        return self._ot_msg_flag("00", 8 + 3) if _OTB_MODE else super().flame_active
+    def cooling_active(self) -> Optional[bool]:  # 3220|00
+        return self._ot_msg_flag("00", 8 + 4)
 
     @property
-    def cooling_active(self) -> Optional[bool]:  # 3220/00
-        return self._ot_msg_flag(
-            "00", 8 + 4
-        )  # if _OTB_MODE else None  # TODO: super...
+    def cooling_enabled(self) -> Optional[bool]:  # 3220|00
+        return self._ot_msg_flag("00", 2)
 
     @property
-    def cooling_enabled(self) -> Optional[bool]:  # 3220/00
-        return self._ot_msg_flag("00", 2)  # if _OTB_MODE else None  # TODO: super().xxx
-
-    @property
-    def fault_present(self) -> Optional[bool]:  # 3220/00
-        return self._ot_msg_flag("00", 8)  # if _OTB_MODE else None  # TODO: super().xxx
+    def fault_present(self) -> Optional[bool]:  # 3220|00
+        return self._ot_msg_flag("00", 8)
 
     @property
     def opentherm_schema(self) -> dict:
@@ -1107,11 +1129,10 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
             "dhw_flow_rate": self._ot_msg_value("13"),
             "dhw_setpoint": self._ot_msg_value("38"),
             "dhw_temp": self._ot_msg_value("1A"),
+            "oem_code": self._ot_msg_value("73"),
             "outside_temp": self._ot_msg_value("1B"),
             "rel_modulation_level": self.rel_modulation_level_ot,
             #
-            "oem_code": self._ot_msg_value("73"),
-            #
             "ch_active": self._ot_msg_flag("00", 8 + 1),
             "ch_enabled": self._ot_msg_flag("00", 0),
             "cooling_active": self._ot_msg_flag("00", 8 + 4),
@@ -1139,6 +1160,7 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
             "boiler_return_temp": self._msg_value(_3210, key=SZ_TEMPERATURE),
             "boiler_setpoint": self._msg_value(_22D9, key=SZ_SETPOINT),
             "ch_max_setpoint": self._msg_value(_1081, key=SZ_SETPOINT),
+            "ch_setpoint": self._msg_value(_3EF0, key="ch_setpoint"),
             "ch_water_pressure": self._msg_value(_1300, key=SZ_PRESSURE),
             "dhw_flow_rate": self._msg_value(_12F0, key="dhw_flow_rate"),
             "dhw_setpoint": self._msg_value(_1300, key=SZ_SETPOINT),
@@ -1146,11 +1168,10 @@ class OtbGateway(Actuator, HeatDemand):  # OTB (10): 3220 (22D9, others)
             "outside_temp": self._msg_value(_1290, key=SZ_TEMPERATURE),
             "rel_modulation_level": self.rel_modulation_level,
             #
-            "ch_active": self.ch_active,
-            "ch_enabled": self.ch_enabled,
-            "ch_setpoint": self.ch_setpoint,
-            "dhw_active": self.dhw_active,
-            "flame_active": self.flame_active,
+            "ch_active": self._msg_value(_3EF0, key="ch_active"),
+            "ch_enabled": self._msg_value(_3EF0, key="ch_enabled"),
+            "dhw_active": self._msg_value(_3EF0, key="dhw_active"),
+            "flame_active": self._msg_value(_3EF0, key="flame_active"),
         }
 
     @property
