@@ -8,7 +8,7 @@
 # - Ierlandfan: 3150, 31D9, 31DA, others
 # - ReneKlootwijk: 3EF0
 # - brucemiranda: 3EF0, others
-# - janvken: 1470, 1F70, 22B0, 2411, others
+# - janvken: 10D0, 1470, 1F70, 22B0, 2411, several others
 
 import logging
 import re
@@ -104,6 +104,7 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     RP,
     RQ,
     W_,
+    F6,
     F8,
     F9,
     FA,
@@ -179,6 +180,38 @@ def parser_0001(payload, msg) -> Optional[dict]:
     # W/--:/--:/12:/00-0000-0501 = Test transmit
     # W/--:/--:/12:/00-0000-0505 = Field strength
 
+    if payload[2:6] in ("2000", "8000", "A000"):
+        mode = "hvac"
+    elif payload[2:6] in ("0000", "FFFF"):
+        mode = "heat"
+    else:
+        mode = "heat"
+
+    if mode == "hvac":
+        assert payload[:2] == "00", payload[:2]
+        # assert payload[2:4] in ("20", "80", "A0"), payload[2:4]
+        # assert payload[4:6] == "00", payload[4:6]
+        assert payload[8:10] in ("00", "04", "10", "20", "FF"), payload[8:10]
+
+        result = {"payload": payload, "slot_num": payload[6:8]}
+        if msg.len >= 6:
+            result.update({"param_num": payload[10:12]})
+        if msg.len >= 7:
+            result.update({"next_slot_num": payload[12:14]})
+        if msg.len >= 8:
+            result.update(
+                {
+                    "boolean_14": None
+                    if payload[14:16] == "FF"
+                    else bool(int(payload[14:16]))
+                }
+            )
+        return result
+
+    assert payload[:2] in ("00",), payload[:2]
+    assert payload[2:6] in ("0000", "FFFF"), payload[2:6]
+    assert payload[8:10] in ("00", "02", "05"), payload[8:10]
+
     return {
         SZ_PAYLOAD: "-".join((payload[:2], payload[2:6], payload[6:8], payload[8:])),
     }
@@ -269,6 +302,14 @@ def parser_0008(payload, msg) -> Optional[dict]:
     # https://www.domoticaforum.eu/viewtopic.php?f=7&t=5806&start=105#p73681
     # e.g. Electric Heat Zone
 
+    #  I --- 01:145038 --:------ 01:145038 0008 002 0314
+    #  I --- 01:145038 --:------ 01:145038 0008 002 F914
+    #  I --- 01:054173 --:------ 01:054173 0008 002 FA00
+    #  I --- 01:145038 --:------ 01:145038 0008 002 FC14
+
+    # RP --- 13:109598 18:199952 --:------ 0008 002 0000
+    # RP --- 13:109598 18:199952 --:------ 0008 002 00C8
+
     if msg.src.type == DEV_TYPE_MAP.JST and msg.len == 13:  # Honeywell Japser, DEX
         assert msg.len == 13, "expecting length 13"
         return {
@@ -276,7 +317,7 @@ def parser_0008(payload, msg) -> Optional[dict]:
             "blob": payload[8:],
         }
 
-    return {SZ_RELAY_DEMAND: percent(payload[2:4])}
+    return {SZ_RELAY_DEMAND: percent(payload[2:4])}  # 3EF0[2:4], 3EF1[10:12]
 
 
 @parser_decorator  # relay_failsafe
@@ -514,40 +555,69 @@ def parser_01e9(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # zone_schedule (fragment)
 def parser_0404(payload, msg) -> Optional[dict]:
-    # Retreival of Zone schedule (NB: 20)
-    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-20-0008-00-0100
-    # RP --- 01:037519 30:185469 --:------ 0404 048 00-20-0008-29-0103-62...
-    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-20-0008-00-0203
-    # RP --- 01:037519 30:185469 --:------ 0404 048 00-20-0008-29-0203-4D...
-    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-20-0008-00-0303
-    # RP --- 01:037519 30:185469 --:------ 0404 038 00-20-0008-1F-0303-C1...
+    # Retreival of Zone schedule (NB: 200008)
+    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-200008-00-0100
+    # RP --- 01:037519 30:185469 --:------ 0404 048 00-200008-29-0103-6E2...
+    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-200008-00-0203
+    # RP --- 01:037519 30:185469 --:------ 0404 048 00-200008-29-0203-4FD...
+    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-200008-00-0303
+    # RP --- 01:037519 30:185469 --:------ 0404 038 00-200008-1F-0303-C10...
 
-    # Retreival of DHW schedule (NB: 23)
-    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-23-0008-00-0100
-    # RP --- 01:037519 30:185469 --:------ 0404 048 00-23-0008-29-0103-68...
-    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-23-0008-00-0203
-    # RP --- 01:037519 30:185469 --:------ 0404 048 00-23-0008-29-0203-ED...
-    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-23-0008-00-0303
-    # RP --- 01:037519 30:185469 --:------ 0404 014 00-23-0008-07-0303-1F...
+    # Retreival of DHW schedule (NB: 230008)
+    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-230008-00-0100
+    # RP --- 01:037519 30:185469 --:------ 0404 048 00-230008-29-0103-618...
+    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-230008-00-0203
+    # RP --- 01:037519 30:185469 --:------ 0404 048 00-230008-29-0203-ED6...
+    # RQ --- 30:185469 01:037519 --:------ 0404 007 00-230008-00-0303
+    # RP --- 01:037519 30:185469 --:------ 0404 014 00-230008-07-0303-13F...
 
-    def _context(seqx) -> dict:
+    # Write a Zone schedule...
+    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0104-688...
+    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0104
+    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0204-007...
+    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0204
+    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0304-8DD...
+    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0304
+    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-11-0404-970...
+    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-11-0400
+
+    # RP --- 01:145038 18:013393 --:------ 0404 007 00-230008-00-01FF  # no schedule
+
+    assert payload[4:6] in ("00", payload[:2]), _INFORM_DEV_MSG
+
+    if int(payload[8:10], 16) * 2 != len(payload[14:]):
+        raise InvalidPayloadError(f"Incorrect fragment length: 0x{payload[8:10]}")
+
+    if msg.verb == RQ:  # have a ctx: idx|frag_idx
         return {
-            SZ_FRAG_NUMBER: int(seqx[10:12], 16),
-            SZ_TOTAL_FRAGS: int(seqx[12:], 16),
-            SZ_FRAG_LENGTH: int(seqx[8:10], 16),
+            SZ_FRAG_NUMBER: int(payload[10:12], 16),
+            SZ_TOTAL_FRAGS: None if payload[12:14] == "00" else int(payload[12:14], 16),
         }
 
-    if msg.verb == RQ:  # RQs have a context: index|fragment_idx
-        return _context(payload)
+    if msg.verb == I_:  # have a ctx: idx|frag_idx
+        return {
+            SZ_FRAG_NUMBER: int(payload[10:12], 16),
+            SZ_TOTAL_FRAGS: int(payload[12:14], 16),
+            SZ_FRAG_LENGTH: None if payload[8:10] == "00" else int(payload[8:10], 16),
+        }
+
+    if payload[12:14] == FF:
+        return {
+            SZ_FRAG_NUMBER: int(payload[10:12], 16),
+            SZ_TOTAL_FRAGS: None,
+        }
 
     return {
-        **_context(payload[:14]),
+        SZ_FRAG_NUMBER: int(payload[10:12], 16),
+        SZ_TOTAL_FRAGS: int(payload[12:14], 16),
+        SZ_FRAG_LENGTH: None if payload[8:10] == "FF" else int(payload[8:10], 16),
         SZ_FRAGMENT: payload[14:],
     }
 
 
 @parser_decorator  # system_fault
 def parser_0418(payload, msg) -> Optional[dict]:
+    # RP --- 01:145038 18:013393 --:------ 0418 022 000000B006F604000000711607697FFFFF7000348A86  # COMMS FAULT, CHANGEOVER
     # RP --- 01:145038 18:013393 --:------ 0418 022 000000B0000000000000000000007FFFFF7000000000  # noqa: E501
     # RP --- 01:145038 18:013393 --:------ 0418 022 000036B0010000000000108000007FFFFF7000000000  # noqa: E501
     # RP --- 01:145038 18:013393 --:------ 0418 022 000000B00401010000008694A3CC7FFFFF70000ECC8A  # noqa: E501
@@ -567,7 +637,7 @@ def parser_0418(payload, msg) -> Optional[dict]:
         assert payload[12:14] in FAULT_DEVICE_CLASS, f"device class: {payload[12:14]}"
         # 1C: 'Comms fault, Actuator': seen with boiler relays
         assert int(payload[10:12], 16) < msg._gwy.config.max_zones or (
-            payload[10:12] in ("1C", F9, FA, FC)
+            payload[10:12] in ("1C", F6, F9, FA, FC)
         ), f"domain id: {payload[10:12]}"
     except AssertionError as exc:
         _LOGGER.warning(
@@ -641,23 +711,29 @@ def parser_0b04(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # mixvalve_config (zone), NB: mixvalves are listen-only
+@parser_decorator  # mixvalve_config (zone), FAN
 def parser_1030(payload, msg) -> Optional[dict]:
+    #  I --- 01:145038 --:------ 01:145038 1030 016 0A-C80137-C9010F-CA0196-CB0100-CC0101
+    #  I --- --:------ --:------ 12:144017 1030 016 01-C80137-C9010F-CA0196-CB010F-CC0101
+    # RP --- 32:155617 18:005904 --:------ 1030 007 00-200100-21011F
+
     def _parser(seqx) -> dict:
         assert seqx[2:4] == "01", seqx[2:4]
 
         param_name = {
+            "20": "unknown_20",  # HVAC
+            "21": "unknown_21",  # HVAC
             "C8": "max_flow_setpoint",  # 55 (0-99) C
             "C9": "min_flow_setpoint",  # 15 (0-50) C
             "CA": "valve_run_time",  # 150 (0-240) sec, aka actuator_run_time
             "CB": "pump_run_time",  # 15 (0-99) sec
-            "CC": f"_{SZ_UNKNOWN}_0",  # ?boolean?
+            "CC": "boolean_cc",  # ?boolean?
         }[seqx[:2]]
 
         return {param_name: int(seqx[4:], 16)}
 
-    assert msg.len == 1 + 5 * 3, msg.len
-    assert payload[30:] in ("00", "01"), payload[30:]
+    assert (msg.len - 1) / 3 in (2, 5), msg.len
+    # assert payload[30:] in ("00", "01"), payload[30:]
 
     params = [_parser(payload[i : i + 6]) for i in range(2, len(payload), 6)]
     return {k: v for x in params for k, v in x.items()}
@@ -771,6 +847,32 @@ def parser_10b0(payload, msg) -> Optional[dict]:
     }
 
 
+@parser_decorator  # filter_change, HVAC
+def parser_10d0(payload, msg) -> Optional[dict]:
+
+    # 2022-07-03T22:52:34.571579 045  W --- 37:171871 32:155617 --:------ 10D0 002 00FF
+    # 2022-07-03T22:52:34.596526 066  I --- 32:155617 37:171871 --:------ 10D0 006 0047B44F0000
+    # then...
+    # 2022-07-03T23:14:23.854089 000 RQ --- 37:155617 32:155617 --:------ 10D0 002 0000
+    # 2022-07-03T23:14:23.876088 084 RP --- 32:155617 37:155617 --:------ 10D0 006 00B4B4C80000
+
+    # 00-FF resets the counter, 00-47-B4-4F-0000 is the value (71 180 79).
+    # Default is 180 180 200. The returned value is the amount of days (180),
+    # total amount of days till change (180), percentage (200)
+
+    if msg.verb == W_:
+        result = {"reset_counter": payload[2:4] == "FF"}
+    else:
+        result = {"days_remaining": int(payload[2:4], 16)}
+
+    if msg.len >= 3:
+        result.update({"days_lifetime": int(payload[4:6], 16)})
+    if msg.len >= 4:
+        result.update({"percent_remaining": percent(payload[6:8])})
+
+    return result
+
+
 @parser_decorator  # device_info
 def parser_10e0(payload, msg) -> Optional[dict]:
     if payload == "00":  # some HVAC devices wil RP|10E0|00
@@ -1086,11 +1188,12 @@ def parser_1f41(payload, msg) -> Optional[dict]:
         payload[6:12] == "FFFFFF"
     ), f"{msg!r}: expected FFFFFF instead of '{payload[6:12]}'"
 
-    result = {
-        "active": {"00": False, "01": True, "FF": None}[payload[2:4]],
-        SZ_MODE: ZON_MODE_MAP.get(payload[4:6]),
-    }
-    if payload[4:6] == ZON_MODE_MAP.TEMPORARY:  # temporary_override
+    result = {SZ_MODE: ZON_MODE_MAP.get(payload[4:6])}
+    if payload[2:4] != "FF":
+        result["active"] = {"00": False, "01": True, "FF": None}[payload[2:4]]
+    # if payload[4:6] == ZON_MODE_MAP.COUNTDOWN:
+    #     result[SZ_UNTIL] = dtm_from_hex(payload[6:12])
+    if payload[4:6] == ZON_MODE_MAP.TEMPORARY:
         result[SZ_UNTIL] = dtm_from_hex(payload[12:24])
 
     return result
@@ -1100,19 +1203,23 @@ def parser_1f41(payload, msg) -> Optional[dict]:
 def parser_1F70(payload, msg) -> Optional[dict]:
     # Seen on Orcon: see 1470, 1F70, 22B0
 
-    assert payload[:2] == "00", _INFORM_DEV_MSG
-    assert payload[2:4] in ("00", "01"), _INFORM_DEV_MSG
-    assert payload[4:8] == "0800", _INFORM_DEV_MSG
-    assert payload[10:14] == "0000", _INFORM_DEV_MSG
-    assert msg.verb in (RQ, W_) or payload[14:16] == "15", _INFORM_DEV_MSG
-    assert msg.verb in (I_, RP) or payload[14:16] == "00", _INFORM_DEV_MSG
-    assert msg.verb == RQ or payload[22:24] == "60", _INFORM_DEV_MSG
-    assert msg.verb != RQ or payload[22:24] == "00", _INFORM_DEV_MSG
-    assert msg.verb == RP or payload[26:] == "000000", _INFORM_DEV_MSG
-    assert msg.verb != RP or payload[26:] == "008000", _INFORM_DEV_MSG
+    try:
+        assert payload[:2] == "00", f"expected 00, not {payload[:2]}"
+        assert payload[2:4] in ("00", "01"), f"expected (00|01), not {payload[2:4]}"
+        assert payload[4:8] == "0800", f"expected 0800, not {payload[4:8]}"
+        assert payload[10:14] == "0000", f"expected 0000, not {payload[10:14]}"
+        assert msg.verb in (RQ, W_) or payload[14:16] == "15"
+        assert msg.verb in (I_, RP) or payload[14:16] == "00"
+        assert msg.verb == RQ or payload[22:24] == "60"
+        assert msg.verb != RQ or payload[22:24] == "00"
+        assert msg.verb == RQ or payload[24:26] in ("E4", "E5", "E6"), _INFORM_DEV_MSG
+        assert msg.verb == RP or payload[26:] == "000000"
+        assert msg.verb != RP or payload[26:] == "008000"
 
-    # assert int(payload[16:18], 16) < 7, _INFORM_DEV_MSG
-    assert msg.verb == RQ or payload[24:26] in ("E4", "E5", "E6"), _INFORM_DEV_MSG
+    except AssertionError as exc:
+        _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
+
+        # assert int(payload[16:18], 16) < 7, _INFORM_DEV_MSG
 
     return {
         "day_idx": payload[16:18],  # depends upon 1470[3:4]?
@@ -1130,6 +1237,8 @@ def parser_1F70(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # rf_bind
 def parser_1fc9(payload, msg) -> list:
+    #  I --- 01:145038 --:------ 01:145038 1FC9 012 F6-2D49-06368E F6-1FC9-06368E
+
     #  I is missing?
     #  W --- 10:048122 01:145038 --:------ 1FC9 006 003EF028BBFA
     #  I --- 01:145038 10:048122 --:------ 1FC9 006 00FFFF06368E
@@ -1169,6 +1278,7 @@ def parser_1fc9(payload, msg) -> list:
             "67",
             "6C",
             "90",
+            F6,
             F9,
             FA,
             FB,
@@ -1520,10 +1630,11 @@ def parser_2389(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # unknown_2400, from OTB
+@parser_decorator  # unknown_2400, from OTB, FAN
 def parser_2400(payload, msg) -> Optional[dict]:
-
-    assert payload == "0000000F", _INFORM_DEV_MSG
+    # RP --- 32:155617 18:005904 --:------ 2400 045 00001111-1010929292921110101020110010000080100010100000009191111191910011119191111111111100  # Orcon FAN
+    # RP --- 10:048122 18:006402 --:------ 2400 004 0000000F
+    # assert payload == "0000000F", _INFORM_DEV_MSG
 
     return {
         SZ_PAYLOAD: payload,
@@ -1548,10 +1659,15 @@ def parser_2401(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # unknown_2410, from OTB
+@parser_decorator  # unknown_2410, from OTB, FAN
 def parser_2410(payload, msg) -> Optional[dict]:
+    # RP --- 10:048122 18:006402 --:------ 2410 020 000000-0000-00000000-00000001-00000001-00-000C  # OTB
+    # RP --- 32:155617 18:005904 --:------ 2410 020 000000-3EE8-00000000-FFFFFFFF-00000000-10-02A6  # Orcon Fan
 
-    assert payload == "00" * 12 + "010000000100000C", _INFORM_DEV_MSG
+    assert payload[:6] == "00" * 3, _INFORM_DEV_MSG
+    assert payload[10:18] == "00" * 4, _INFORM_DEV_MSG
+    assert payload[18:26] in ("00000001", "FFFFFFFF"), _INFORM_DEV_MSG
+    assert payload[26:34] in ("00000001", "00000000"), _INFORM_DEV_MSG
 
     return {
         SZ_PAYLOAD: payload,
@@ -1566,15 +1682,18 @@ def parser_2411(payload, msg) -> Optional[dict]:
     # RQ --- 37:171871 32:155617 --:------ 2411 003 00004E            # 11th menu option (i.e. 0x0A)
     # RP --- 32:155617 37:171871 --:------ 2411 023 00004E460000000001000000000000000100000001A600
 
-    def counter(x):
+    def counter(x) -> int:
         return int(x, 16)
 
+    def centile(x) -> float:
+        return int(x, 16) / 10
+
     _2411_DATA_TYPES = {
         "00": (2, counter),  # 4E (0-1), 54 (15-60)
-        "01": (2, counter),  # 52 (0-250) PIR ?should be percent
-        "0F": (2, percent),  # xx (0.0-1.0)
-        "10": (4, counter),  # 31 (0-1800)
-        "92": (4, temp_from_hex),  # 75 (0-30)
+        "01": (2, centile),  # 52 (0.0-25.0) (%)
+        "0F": (2, percent),  # xx (0.0-1.0) (%)
+        "10": (4, counter),  # 31 (0-1800) (days)
+        "92": (4, temp_from_hex),  # 75 (0-30) (C)
     }  # TODO: _2411_TYPES.get(payload[8:10], (8, no_op))
 
     assert (
@@ -1621,13 +1740,13 @@ def parser_2420(payload, msg) -> Optional[dict]:
     }
 
 
-@parser_decorator  # _state (of unknown), from hometronics controller
+@parser_decorator  # _state (of cooling?), from BDR91T, hometronics CTL
 def parser_2d49(payload, msg) -> dict:
 
-    assert payload[2:] in ("0000", "C800"), _INFORM_DEV_MSG
+    assert payload[2:] in ("0000", "00FF", "C800", "C8FF"), _INFORM_DEV_MSG
 
     return {
-        "_state": bool_from_hex(payload[2:4]),
+        "state": bool_from_hex(payload[2:4]),
     }
 
 
@@ -1653,10 +1772,16 @@ def parser_2e04(payload, msg) -> Optional[dict]:
         # msg.len in (8, 16)  # evohome 8, hometronics 16
         assert False, f"Packet length is {msg.len} (expecting 8, 16)"
 
-    return {
-        SZ_SYSTEM_MODE: SYS_MODE_MAP[payload[:2]],
-        SZ_UNTIL: dtm_from_hex(payload[2:14]) if payload[14:16] != "00" else None,
-    }  # TODO: double-check the final "00"
+    result = {SZ_SYSTEM_MODE: SYS_MODE_MAP[payload[:2]]}
+    if payload[:2] not in (
+        SYS_MODE_MAP.AUTO,
+        SYS_MODE_MAP.HEAT_OFF,
+        SYS_MODE_MAP.AUTO_WITH_RESET,
+    ):
+        result.update(
+            {SZ_UNTIL: dtm_from_hex(payload[2:14]) if payload[14:16] != "00" else None}
+        )
+    return result  # TODO: double-check the final "00"
 
 
 @parser_decorator  # presence_detect, HVAC sensor
@@ -1790,18 +1915,19 @@ def parser_31d9(payload, msg) -> Optional[dict]:
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
 
-    # bitmap = int(payload[2:4], 16)
+    bitmap = int(payload[2:4], 16)
 
+    # NOTE: 31D9[4:6] is fan_speed (itho?) *or* fan_mode (orcon?)
+    # TODO: 31D9[4:6] == 31DA[38:40] in some systems
+    # TODO: 31D9[4:6] == 31DA[36:38]??
     result = {
-        SZ_EXHAUST_FAN_SPEED: percent(
-            payload[4:6], high_res=True
-        ),  # NOTE: is 31DA/payload[38:40]
+        SZ_EXHAUST_FAN_SPEED: percent(payload[4:6], high_res=True),
         "fan_mode": payload[4:6],
-        # "passive": bool(bitmap & 0x02),
-        # "damper_only": bool(bitmap & 0x04),
-        # "filter_dirty": bool(bitmap & 0x20),
-        # "frost_cycle": bool(bitmap & 0x40),
-        # "has_fault": bool(bitmap & 0x80),
+        "passive": bool(bitmap & 0x02),
+        "damper_only": bool(bitmap & 0x04),
+        "filter_dirty": bool(bitmap & 0x20),
+        "frost_cycle": bool(bitmap & 0x40),
+        "has_fault": bool(bitmap & 0x80),
         "flags": flag8(payload[2:4]),
     }
 
@@ -1809,7 +1935,7 @@ def parser_31d9(payload, msg) -> Optional[dict]:
         return result
 
     try:
-        assert payload[6:8] in ("00", "FE"), f"byte 3: {payload[6:8]}"
+        assert payload[6:8] in ("00", "07", "0A", "FE"), f"byte 3: {payload[6:8]}"
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
 
@@ -1869,8 +1995,6 @@ def parser_31da(payload, msg) -> Optional[dict]:
         0x1F: "-unknown 0x1F-",
     }
 
-    # I --- 37:261128 --:------ 37:261128 31DA 029 00004007D045EF7FFF7FFF7FFF7FFFF808EF03C8000000EFEF7FFF7FFF  # # event-driven?
-    # I --- 37:053679 --:------ 37:053679 31DA 030 00EF007FFF41EF7FFF7FFF7FFF7FFFF800EF0134000000EFEF7FFF7FFF00  # periodic?
     try:
         # assert (
         #     int(payload[2:4], 16) <= 200
@@ -1911,7 +2035,7 @@ def parser_31da(payload, msg) -> Optional[dict]:
     # 12 Fan supply speed (%)                 SZ_SUPPLY_FAN_SPEED
     # 13 Remaining after run time (humidity scenario) (min.)  SZ_REMAINING_TIME
     # 14 Preheater control (MaxComfort) (%)   SZ_PRE_HEAT
-    # 16 Actual supply flow rate (m3/h)       SZ_SUPPLY_FLOW
+    # 16 Actual supply flow rate (m3/h)       SZ_SUPPLY_FLOW (Orcon is m3/h, others L/s)
     # 17 Current discharge flow rate (m3/h)   SZ_EXHAUST_FLOW
 
     return {
@@ -1933,8 +2057,8 @@ def parser_31da(payload, msg) -> Optional[dict]:
         SZ_SUPPLY_FAN_SPEED: percent(payload[40:42]),
         SZ_POST_HEAT: percent(payload[46:48], high_res=False),
         SZ_PRE_HEAT: percent(payload[48:50], high_res=False),
-        SZ_SUPPLY_FLOW: double(payload[50:54], factor=100),  # L/sec or m^3/hr
-        SZ_EXHAUST_FLOW: double(payload[54:58], factor=100),  # L/sec or m^3/hr
+        SZ_SUPPLY_FLOW: double(payload[50:54], factor=100),  # L/sec
+        SZ_EXHAUST_FLOW: double(payload[54:58], factor=100),  # L/sec
     }
 
 
@@ -2073,11 +2197,12 @@ def parser_3220(payload, msg) -> Optional[dict]:
     return result
 
 
-@parser_decorator  # unknown_3221, from OTB
+@parser_decorator  # unknown_3221, from OTB, FAN
 def parser_3221(payload, msg) -> Optional[dict]:
 
-    # 2021-11-03T09:55:43.112792 071 RP --- 10:052644 18:198151 --:------ 3221 002 000F
-    # 2021-11-02T05:15:55.767108 046 RP --- 10:048122 18:006402 --:------ 3221 002 0000
+    # RP --- 10:052644 18:198151 --:------ 3221 002 000F
+    # RP --- 10:048122 18:006402 --:------ 3221 002 0000
+    # RP --- 32:155617 18:005904 --:------ 3221 002 000A
 
     assert int(payload[2:], 16) <= 0xC8, _INFORM_DEV_MSG
 
@@ -2173,7 +2298,7 @@ def parser_3ef0(payload, msg) -> dict:
         mod_level = percent(payload[2:4], high_res=False)
 
     result = {
-        "modulation_level": mod_level,
+        "modulation_level": mod_level,  # 0008[2:4], 3EF1[10:12]
         "_flags_2": payload[4:6],
     }
 
@@ -2268,7 +2393,7 @@ def parser_3ef1(payload, msg) -> dict:
     cycle_countdown = None if payload[2:6] == "7FFF" else int(payload[2:6], 16)
 
     return {
-        "modulation_level": percent(payload[10:12]),
+        "modulation_level": percent(payload[10:12]),  # 0008[2:4], 3EF0[2:4]
         "actuator_countdown": int(payload[6:10], 16),
         "cycle_countdown": cycle_countdown,
         f"_{SZ_UNKNOWN}_0": payload[12:],
