@@ -13,6 +13,11 @@ connection: &con00
   connector: serialdev,/dev/ttyUSB0,115200n81,local
   options:
     max-connections: 3
+
+For socat, see:
+  socat -dd pty,raw,echo=0 pty,raw,echo=0
+  python client.py monitor /dev/pts/0
+  cat packet.log | cut -d ' ' -f 2- | unix2dos > /dev/pts/1
 """
 
 # TODO: switch dtm from naive to aware
@@ -906,11 +911,13 @@ class PacketProtocolQos(PacketProtocolPort):
         if self._qos_cmd:
             if pkt._hdr == self._qos_cmd.tx_header:
                 if self._tx_rcvd:
-                    _LOGGER.error(f"already seen tx_rcvd, rx_rcvd={self._rx_rcvd}")
+                    err = f"have seen tx_rcvd({self._tx_rcvd}), rx_rcvd={self._rx_rcvd}"
+                    _LOGGER.error(err)
                 self._tx_rcvd = pkt
-            if pkt._hdr == self._qos_cmd.rx_header:
+            elif pkt._hdr == self._qos_cmd.rx_header:
                 if self._rx_rcvd:
-                    _LOGGER.error(f"already seen rx_rcvd, tx_rcvd={self._tx_rcvd}")
+                    err = f"have seen rx_rcvd({self._rx_rcvd}), tx_rcvd={self._tx_rcvd}"
+                    _LOGGER.error(err)
                 self._rx_rcvd = pkt
 
         super()._pkt_received(pkt)
